import { select, pointer } from 'd3';
import { viz } from './viz';
import data from './data.csv';

for (const d of data) {
  d.Total = +d.Total;
  d.HP = +d.HP;
  d.Attack = +d.Attack;
  d.Defense = +d.Defense;
  d['Sp. Atk'] = +d['Sp. Atk'];
  d['Sp. Def'] = +d['Sp. Def'];
  d.Speed = +d.Speed;
  d.Generation = +d.Generation;
  d.Legendary = d.Legendary === 'True';
}

export const main = (container, { state, setState }) => {
  if (state.width === undefined) {
    const resizeObserver = new ResizeObserver((entries) => {
      const { width, height } = entries[0].contentRect;
      setState((state) => ({ ...state, width, height }));
    });
    resizeObserver.observe(container);
    return;
  }

  const { width, height } = state;

  const svg = select(container)
    .selectAll('svg')
    .data([null])
    .join('svg')
    .attr('width', width)
    .attr('height', height)
    .on('mousemove', (event) => {
      const [hoveredX, hoveredY] = pointer(event);
      setState((state) => ({
        ...state,
        hoveredX,
        hoveredY,
      }));
    });

  let tooltip = select('body').select('.tooltip');
  if (tooltip.empty()) {
    tooltip = select('body').append('div')
        .attr('class', 'tooltip')
        .style('opacity', 0)
        .style('position', 'absolute')
        .style('padding', '10px')
        .style('background', 'rgba(255, 255, 255, 0.8)')
        .style('border', '1px solid #ddd')
        .style('border-radius', '5px')
        .style('pointer-events', 'none')
        .style('font-size', '12px')
        .style('display', 'none');
  }

  viz(svg, {
    data,
    xValue: (d) => d.Attack,
    xAxisLabelText: 'Attack',
    xAxisLabelOffset: 38,
    yValue: (d) => d.Defense,
    yAxisLabelText: 'Defense',
    yAxisLabelOffset: 17,
    innerRectFill: '#E8E8E8',
    circleRadius: 153 / 20,
    circleOpacity: 629 / 1000,
    marginTop: 20,
    marginBottom: 50,
    marginLeft: 54,
    marginRight: 20,
    width,
    height,
    hoveredX: state.hoveredX,
    hoveredY: state.hoveredY,
    tooltip,
  });
};
