import { select, d3, scaleLinear, axisBottom, axisLeft, scaleOrdinal, schemeDark2, extent } from 'd3';
import { axes } from './axes';

export const viz = (
  svg,
  {
    data,
    xValue,
    xAxisLabelText,
    xAxisLabelOffset,
    yValue,
    yAxisLabelText,
    yAxisLabelOffset,
    marginLeft,
    marginTop,
    marginRight,
    marginBottom,
    width,
    height,
    innerRectFill,
    circleRadius,
    circleOpacity,
    hoveredX,
    hoveredY,
    tooltip,
  }
) => {
  svg.selectAll('*').remove();
  //Temporary SVG for measuring and subtracting legend width, is there a better way to do this?
  const tempSvg = select(document.body).append('svg').style('visibility', 'hidden');
  const colorScale = scaleOrdinal(schemeDark2).domain(data.map(d => d.Generation));
  const tempLegend = tempSvg.append('g');
  colorScale.domain().forEach((gen, index) => {
    const legendRow = tempLegend.append('g')
      .attr('transform', `translate(0, ${index * 20})`);
    legendRow.append('circle')
      .attr('cx', 0)
      .attr('cy', 0)
      .attr('r', 5)
      .attr('fill', colorScale(gen));
    legendRow.append('text')
      .attr('x', 15)
      .attr('y', 5)
      .text(`Generation ${gen}`);
  });
  const legendWidth = tempLegend.node().getBBox().width;
  marginRight += legendWidth;
  tempSvg.remove();

  const xScale = scaleLinear().domain(extent(data, xValue)).range([marginLeft, width - marginRight]);
  const yScale = scaleLinear().domain(extent(data, yValue)).range([height - marginBottom, marginTop]);

  svg.append('rect')
    .attr('x', marginLeft)
    .attr('y', marginTop)
    .attr('width', width - marginLeft - marginRight)
    .attr('height', height - marginTop - marginBottom)
    .attr('fill', innerRectFill);

  axes(svg, {
    width,
    height,
    xScale,
    yScale,
    xAxisLabelText,
    xAxisLabelOffset,
    yAxisLabelText,
    yAxisLabelOffset,
    marginLeft,
    marginBottom,
  });

   svg.selectAll('circle').data(data).join('circle')
    .attr('cx', d => xScale(xValue(d)))
    .attr('cy', d => yScale(yValue(d)))
    .attr('r', circleRadius)
    .attr('fill', d => colorScale(d.Generation))
    .attr('opacity', circleOpacity)
    .on('mouseover', (event, d) => {
      tooltip
        .style('display', 'block')
        .style('opacity', 1)
        .html(`Attack: ${d.Attack}<br>Defense: ${d.Defense}`)
        .style('left', `${event.pageX + 10}px`)
        .style('top', `${event.pageY + 10}px`);
    })
    .on('mouseout', () => {
      tooltip
        .style('opacity', 0)
        .end()
        .then(() => tooltip.style('display', 'none'));
    });

  const legend = svg.append('g')
    .attr('transform', `translate(${width - marginRight + 20}, ${marginTop})`);
  colorScale.domain().forEach((gen, index) => {
    const legendRow = legend.append('g')
      .attr('transform', `translate(0, ${index * 20})`);
    legendRow.append('circle')
      .attr('cx', 0)
      .attr('cy', 10)
      .attr('r', 5)
      .attr('fill', colorScale(gen));
    legendRow.append('text')
      .attr('x', 10)
      .attr('y', 15)
      .text(`Generation ${gen}`);
  });
  
  const [xMin, xMax] = xScale.range();
  const [yMax, yMin] = yScale.range();

  const showHoveredLines =
    hoveredX < xMax &&
    hoveredX > xMin &&
    hoveredY < yMax &&
    hoveredY > yMin;

  const x = hoveredX;
  const y = hoveredY;
  const hoveredLinesData = showHoveredLines
    ? [
        { x1: x, y1: y, x2: x, y2: yMax },
        { x1: xMin, y1: y, x2: x, y2: y },
      ]
    : [];

  svg
    .selectAll('line.hovered-line')
    .data(hoveredLinesData)
    .join('line')
    .attr('class', 'hovered-line')
    .attr('x1', (d) => d.x1)
    .attr('y1', (d) => d.y1)
    .attr('x2', (d) => d.x2)
    .attr('y2', (d) => d.y2)
    .attr('stroke', 'black');
};